#include"font.hpp"
#include"eyesquat_yoyo.hpp"
#include"screen.hpp"




#define PROGNAME  "eyesquat"


namespace{


bool  loop_flag;


size_t  speed = 1;


std::vector<ChunkBlock>
blocks;




void
update_screen()
{
  SDL_SetRenderDrawColor(renderer,0,0,0,255);

  SDL_RenderClear(renderer);

  SDL_SetRenderDrawColor(renderer,255,255,255,255);

    if(blocks.size())
    {
      int  x = 0;

        for(auto&  blk: blocks)
        {
            if(yoyo::draw(blk,x))
            {
              break;
            }
        }
    }


  SDL_RenderPresent(renderer);
}


void
process_keyup(const SDL_KeyboardEvent&  evt)
{
    switch(evt.keysym.sym)
    {
      case(SDLK_f):
        change_fullscreen();
        break;
      case(SDLK_z):
          if(speed > 1)
          {
            --speed;
          }
        break;
      case(SDLK_x):
          if(speed < 10)
          {
            ++speed;
          }
        break;
      case(SDLK_ESCAPE):
        loop_flag = false;
        break;
      case(SDLK_F5):
        screenshot(PROGNAME ".bmp");
        break;
    }
}


void
push(char16_t  c)
{
    if((c != '\0') &&
       (c != '\n') &&
       (c != '\r'))
    {
      blocks.emplace_back();

      auto&  blk = blocks.back();

      render(blk,c);
    }
}


void
load_file(const char*  path)
{
  auto  f = fopen(path,"rb");

    if(f)
    {
      blocks.clear();

        for(;;)
        {
          auto  c = fget_utf8(f);

            if(feof(f))
            {
              break;
            }


          push(c);
        }


      fclose(f);
    }
}


void
load_string(const char*  s)
{
  blocks.clear();

    for(;;)
    {
      auto  c = sget_utf8(s);

        if(!c)
        {
          break;
        }


      push(c);
    }
}


void
process()
{
  static SDL_Event  evt;

    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_DROPFILE):
            load_file(evt.drop.file);

            SDL_free(evt.drop.file);
            break;
          case(SDL_KEYUP):
            process_keyup(evt.key);
            break;
          case(SDL_MOUSEBUTTONDOWN):
              if((evt.button.clicks == 2) && (SDL_HasClipboardText() == SDL_TRUE))
              {
                auto  s = SDL_GetClipboardText(); 

                  if(s)
                  {
                    load_string(s);

                    SDL_free(s);
                  }
              }
            break;
          case(SDL_QUIT):
            loop_flag = false;
            break;
        }
    }
}


const char16_t
initial_text[] = u"つめたい水";


}




int
main(int  argc, char**  argv)
{
    if(!open_font("font.ttf"))
    {
      return -1;
    }


  SDL_Init(SDL_INIT_VIDEO);

  window = SDL_CreateWindow(PROGNAME " - " __DATE__,32,32,window_w,window_h,0);

  renderer = SDL_CreateRenderer(window,-1,0);

  loop_flag = true;

    for(auto  c: initial_text)
    {
      push(c);
    }


    while(loop_flag)
    {
      yoyo::step(speed);

      process();

      update_screen();


      SDL_Delay(16);
    }


  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);

  SDL_Quit();

  close_font();

  return 0;
}




